@@ -2,6 +2,7 @@ from .core.STDataset import STDataset
 from .run import run
 
 from .core.io import load_10X, load_anndata_h5, load_table
+from .core.combinations import find_combinations
 from . import filters, normalizers, plot
 
 from ._version import __version__
@@ -15,4 +16,5 @@ __all__ = [
     "filters",
     "normalizers",
     "plot",
+    "find_combinations",
 ]
