@@ -3,9 +3,9 @@ from argparse import ArgumentParser
 from multiprocessing import cpu_count
 from pathlib import Path
 
-from svgbit import load_10X, load_anndata_h5, run, plot
+from svgbit import load_10X, load_anndata_h5, load_table, run, plot
 from svgbit.filters import low_variance_filter, quantile_filter
-from svgbit.normalizers import logcpm_normalizer
+from svgbit.normalizers import logcpm_normalizer, cpm_normalizer
 
 
 def main() -> None:
@@ -19,6 +19,13 @@ def main() -> None:
         help='''Read Spatial Transcriptomics data. Support format in 10X Space
                 Ranger(dir named ``outs``) and anndata hdf5''',
     )
+    parser.add_argument(
+        "--normalization",
+        default=None,
+        help='''apply which normalization on read data. If None (default), neither
+                normalization will apply. Supported values: None, cpm, logcpm
+                (default: %(default)s)''',
+    )
     parser.add_argument(
         "--k",
         type=int,
@@ -45,7 +52,7 @@ def main() -> None:
                 (default: %(default)s)''',
     )
     parser.add_argument(
-        "--savedir",
+        "--save_dir",
         default=".",
         help='''path to save results (default: %(default)s)''',
     )
@@ -60,13 +67,33 @@ def main() -> None:
     read_path = Path(args.read_path)
     if read_path.is_dir():
         load_func = load_10X
+        kwargs = {}
     else:
-        load_func = load_anndata_h5
+        if read_path.suffix in [".h5", ".hdf5", ".h5ad"]:
+            load_func = load_anndata_h5
+            kwargs = {}
+        else:
+            load_func = load_table
+            if ".csv" in read_path.suffixes:
+                sep = ","
+            elif ".tsv" in read_path.suffixes:
+                sep = "\t"
+            kwargs = {"index_col": 0, "header": 0, "sep": sep}
 
-    d = load_func(read_path)
+    d = load_func(read_path, **kwargs)
     d = low_variance_filter(d)
     d = quantile_filter(d, 0.99)
-    d = logcpm_normalizer(d)
+    norm = args.normalization
+    if norm is not None:
+        norm = norm.lower()
+        if norm == "none":
+            pass
+        elif norm == "logcpm":
+            d = logcpm_normalizer(d)
+        elif norm == "cpm":
+            d = cpm_normalizer(d)
+        else:
+            raise ValueError(f"Not supported normalization: {args.normalization}")
     d = run(
         d,
         k=args.k,
@@ -75,7 +102,9 @@ def main() -> None:
         cores=args.cores,
     )
 
-    savedir = Path(args.savedir)
+    savedir = Path(args.save_dir)
+    if not savedir.exists():
+        savedir.mkdir()
     d.hotspot_df.to_csv(Path.joinpath(savedir, "hotspot_df.csv"))
     d.AI.to_csv(Path.joinpath(savedir, "AI.csv"))
     d.Di.to_csv(Path.joinpath(savedir, "Di.csv"))
