@@ -8,7 +8,6 @@ import matplotlib as mpl
 import matplotlib.pyplot as plt
 import pandas as pd
 from PIL import Image
-from scipy.cluster import hierarchy as sch
 from seaborn import color_palette
 
 from .STDataset import STDataset
@@ -18,8 +17,11 @@ def _svg_heatmap(
     hotspot_df: pd.DataFrame,
     coordinate_df: pd.DataFrame,
     cluster_result: pd.Series,
+    spot_type: pd.DataFrame,
     save_path: Union[str, Path],
     he_image: Optional[Union[str, Path]] = None,
+    s: float = 4,
+    dpi: float = 300,
 ) -> None:
     """
     Draw SVG distribution heatmap.
@@ -35,20 +37,31 @@ def _svg_heatmap(
     cluster_result : pd.Series
         A pd.Series for cluster result.
 
+    spot_type : pd.DataFrame
+        A pd.DataFrame for assigned type_df.
+
     save_path : str or pathlib.Path
         Heatmap save path.
 
     he_image : str or pathlib.Path, default None
         H&E image of tissue. If None is given (default), distribution map
         will not show tissue picture.
+
+    s : float, default 4
+        Spot size.
+
+    dpi : float, default 300
+        DPI for saved figure.
     """
 
+    spot_type = spot_type.sort_values(by="type_1")
+
     left, bottom = 0.1, 0.1
     width, height = 0.66, 0.66
     spacing, cluster_width = 0.03, 0.22
     rect_heatmap = [left + spacing, bottom + spacing * 2, width, height]
 
-    fig = plt.figure(figsize=(10, 10), dpi=300)
+    fig = plt.figure(figsize=(10, 10), dpi=dpi)
     ax_heatmap = fig.add_axes(rect_heatmap)
 
     if he_image is not None:
@@ -74,7 +87,7 @@ def _svg_heatmap(
             cmap="autumn_r",
             vmin=0,
             vmax=1,
-            s=4,
+            s=s,
             alpha=0.7,
         )
 
@@ -88,16 +101,9 @@ def _svg_heatmap(
     ax_cb = fig.add_axes(rect_cb)
     fig.colorbar(sc, cax=ax_cb)
 
-    spot_distmat = sch.distance.pdist(hotspot_df, metric="jaccard")
-    Z_spot = sch.linkage(spot_distmat, method="ward")
-    spot_result = pd.Series(
-        sch.fcluster(Z_spot, t=8, criterion="maxclust"),
-        index=hotspot_df.index,
-    ).sort_values()
-
     ax_heatmap.imshow(
         hotspot_df.reindex(
-            index=spot_result.index,
+            index=spot_type.index,
             columns=cluster_result.index,
         ),
         cmap="Reds",
@@ -126,6 +132,8 @@ def _hotspot_distribution_map(
     cluster: Union[str, int],
     save_path: Union[str, Path],
     he_image: Optional[Union[str, Path]] = None,
+    s: float = 4,
+    dpi: float = 300,
 ) -> None:
     """
     Draw hotspot distribution map for one SVG cluster.
@@ -150,9 +158,15 @@ def _hotspot_distribution_map(
     he_image : str or pathlib.Path, default None
         H&E image of tissue. If None is given (default), distribution map
         will not show tissue picture.
+
+    s : float, default 4
+        Spot size.
+
+    dpi : float, default 300
+        DPI for saved figure.
     """
 
-    fig, ax = plt.subplots(figsize=(10, 10), dpi=300)
+    fig, ax = plt.subplots(figsize=(10, 10), dpi=dpi)
     cluster = int(cluster)
 
     ax.set_title(f"Cluster {cluster} hotspot distribution")
@@ -166,7 +180,7 @@ def _hotspot_distribution_map(
         cmap="autumn_r",
         vmin=0,
         vmax=1,
-        s=4,
+        s=s,
         alpha=0.7,
     )
     fig.colorbar(sc)
@@ -186,6 +200,8 @@ def _spot_type_map(
     save_path: Union[str, Path],
     he_image: Optional[Union[str, Path]] = None,
     draw_uncertain: bool = True,
+    s: float = 16,
+    dpi: float = 300,
 ) -> None:
     """
     Draw SVG type map.
@@ -210,6 +226,12 @@ def _spot_type_map(
 
     draw_uncertain : bool, default True
         Whether to draw uncertain spots.
+
+    s : float, default 16
+        Spot size.
+
+    dpi : float, default 300
+        DPI for saved figure.
     """
 
     if draw_uncertain:
@@ -235,11 +257,11 @@ def _spot_type_map(
     else:
         legend_fontsize = mpl.rcParams["legend.title_fontsize"]
 
-    fig, ax = plt.subplots(figsize=(10, 10), dpi=300)
+    fig, ax = plt.subplots(figsize=(10, 10), dpi=dpi)
     sc = ax.scatter(
         coordinate_df["X"],
         coordinate_df["Y"],
-        s=16,
+        s=s,
         c=type_df["type_1"],
         cmap=cmap,
     )
@@ -269,6 +291,8 @@ def svg_heatmap(
     dataset: STDataset,
     save_path: Union[str, Path],
     he_image: Optional[Union[str, Path]] = None,
+    s: float = 4,
+    dpi: float = 300,
 ) -> None:
     """
     Draw SVG distribution heatmap.
@@ -284,13 +308,26 @@ def svg_heatmap(
     he_image : str or pathlib.Path, default None
         H&E image of tissue. If None is given (default), distribution map
         will not show tissue picture.
+
+    s : float, default 4
+        Spot size.
+
+    dpi : float, default 300
+        DPI for saved figure.
     """
+    coor_df = dataset.coordinate_df
+    if he_image is None:
+        if dataset._array_coordinate is not None:
+            coor_df = dataset._array_coordinate
     _svg_heatmap(
-        dataset.hotspot_df,
-        dataset.coordinate_df,
-        dataset.svg_cluster,
-        save_path,
-        he_image,
+        hotspot_df=dataset.hotspot_df,
+        coordinate_df=coor_df,
+        cluster_result=dataset.svg_cluster,
+        spot_type=dataset.spot_type,
+        save_path=save_path,
+        he_image=he_image,
+        s=s,
+        dpi=dpi,
     )
 
 
@@ -299,6 +336,8 @@ def hotspot_distribution_map(
     cluster: Union[str, int],
     save_path: Union[str, Path],
     he_image: Optional[Union[str, Path]] = None,
+    s: float = 4,
+    dpi: float = 300,
 ) -> None:
     """
     Draw hotspot distribution map for one SVG cluster.
@@ -317,14 +356,26 @@ def hotspot_distribution_map(
     he_image : str or pathlib.Path, default None
         H&E image of tissue. If None is given (default), distribution map
         will not show tissue picture.
+
+    s : float, default 4
+        Spot size.
+
+    dpi : float, default 300
+        DPI for saved figure.
     """
+    coor_df = dataset.coordinate_df
+    if he_image is None:
+        if dataset._array_coordinate is not None:
+            coor_df = dataset._array_coordinate
     _hotspot_distribution_map(
-        dataset.hotspot_df,
-        dataset.coordinate_df,
-        dataset.svg_cluster,
-        cluster,
-        save_path,
-        he_image,
+        hotspot_df=dataset.hotspot_df,
+        coordinate_df=coor_df,
+        cluster_result=dataset.svg_cluster,
+        cluster=cluster,
+        save_path=save_path,
+        he_image=he_image,
+        s=s,
+        dpi=dpi,
     )
 
 
@@ -333,6 +384,8 @@ def spot_type_map(
     save_path: Union[str, Path],
     he_image: Optional[Union[str, Path]] = None,
     draw_uncertain: bool = True,
+    s: float = 16,
+    dpi: float = 300,
 ) -> None:
     """
     Draw SVG type map.
@@ -351,12 +404,24 @@ def spot_type_map(
 
     draw_uncertain : bool, default True
         Whether to draw uncertain spots.
+
+    s : float, default 16
+        Spot size.
+
+    dpi : float, default 300
+        DPI for saved figure.
     """
+    coor_df = dataset.coordinate_df
+    if he_image is None:
+        if dataset._array_coordinate is not None:
+            coor_df = dataset._array_coordinate
     _spot_type_map(
-        dataset.hotspot_df,
-        dataset.coordinate_df,
-        dataset.spot_type,
-        save_path,
-        he_image,
-        draw_uncertain,
+        hotspot_df=dataset.hotspot_df,
+        coordinate_df=coor_df,
+        type_df=dataset.spot_type,
+        save_path=save_path,
+        he_image=he_image,
+        draw_uncertain=draw_uncertain,
+        s=s,
+        dpi=dpi,
     )
